# noinspection PyUnresolvedReferences
from flaskel.config import *  # noqa: F403,F401,E0401 pylint: disable=unused-wildcard-import,wildcard-import

LOCALE = "it_IT.utf8"
ERROR_PAGE = "errors.html"
ERROR_HANDLER = "web"
